@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '0.9.8'
+uwsgi_version = '0.9.8.1'
 
 import os
 import re
@@ -26,7 +26,6 @@ if not GCC:
     GCC = 'gcc'
 
 	
-    
 
 def spcall(cmd):
     p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,stderr=open('/dev/null','w'))
@@ -63,8 +62,26 @@ def add_o(x):
     return x
 
 
-def compile(file, objfile, cflags):
-    cmdline = "%s -c %s -o %s %s" % (GCC, cflags, objfile, file)
+def compile(cflags, objfile, srcfile):
+    source_stat = os.stat(srcfile)
+    header_stat = os.stat('uwsgi.h')
+    try:
+        if os.environ.get('UWSGI_FORCE_REBUILD', None):
+            raise
+        object_stat = os.stat(objfile)
+        if object_stat[8] <= source_stat[8]:
+            raise
+        if object_stat[8] <= header_stat[8]:
+            raise
+        for profile in os.listdir('buildconf'):
+            profile_stat = os.stat('buildconf/%s' % profile)
+            if object_stat[8] <= profile_stat[8]:
+                raise
+        print("%s is up to date" % objfile)
+        return
+    except:
+        pass
+    cmdline = "%s -c %s -o %s %s" % (GCC, cflags, objfile, srcfile)
     print(cmdline)
     ret = os.system(cmdline)
     if ret != 0:
@@ -659,7 +676,6 @@ if __name__ == "__main__":
             pass
         if not '/' in bconf:
             bconf = 'buildconf/%s' % bconf
-
         uc = uConf(bconf)
         gcc_list, cflags, ldflags, libs = uc.get_gcll()
         try:
@@ -667,6 +683,12 @@ if __name__ == "__main__":
         except:
             name = None
         build_plugin(sys.argv[2], uc, cflags, ldflags, libs, name)
+    elif cmd == '--clean':
+        os.system("rm *.o")
+        os.system("rm proto/*.o")
+        os.system("rm lib/*.o")
+        os.system("rm plugins/*/*.o")
+
     else:
         print("unknown uwsgiconfig command")
         sys.exit(1)
